import logging
import threading
import traceback

from rest_framework import status
from rest_framework.exceptions import APIException

from .bot import alert_to_telegram
from .responses import CustomResponse
from .translate import default_error_massage


class RESTException(APIException):

    def __init__(self, status_code=400, detail=None, code=None, **kwargs):
        super().__init__(detail, code)
        self.status_code = status_code
        if not detail:
            self.detail = kwargs


def custom_exception_handler(exc, context):
    request = context.get('request')
    language = request.headers.get("language", "en")
    traceback_ = traceback.format_exc()
    logging.error(traceback_)

    try:
        message = dict(getattr(exc, 'detail', exc))
        error_message = message.get('message', default_error_massage.get(language))
    except Exception as error:
        message = str(getattr(exc, 'detail', exc))
        error_message = default_error_massage.get(language)

    threading.Thread(target=alert_to_telegram, args=[traceback_, message]).start()
    return CustomResponse(
        message=error_message,
        status=getattr(exc, 'status_code', status.HTTP_500_INTERNAL_SERVER_ERROR)
    )



from math import sin, cos, sqrt, atan2, radians

# Approximate radius of earth in km
R = 6373.0


def define_distance(user1: dict, user2: dict, unit: str = 'm', user1_to_user2=True, precision=1) -> float:
    if not (user1 and user2):
        return None
    try:
        client_lat = radians(user1.get('lat'))
        client_lot = radians(user1.get('lng'))
        sitter_lat = radians(user2.get('lat'))
        sitter_lot = radians(user2.get('lng'))

        delta_lon = sitter_lot - client_lot if user1_to_user2 else client_lot - sitter_lot
        delta_lat = sitter_lat - client_lat if user1_to_user2 else client_lat - sitter_lat

        a = sin(delta_lat / 2) ** 2 + cos(client_lat) * cos(sitter_lat) * sin(delta_lon / 2) ** 2
        c = 2 * atan2(sqrt(a), sqrt(1 - a))

        distance = R * c

        if unit == 'm':
            return round(distance * 1000)
        elif unit == 'km':
            return round(distance, precision)
    except TypeError:
        return None
